/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */

package eu.DECIDEh2020.architect.plugin.descriptorWizard;


import org.apache.jena.rdf.model.Resource;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Label;
import org.eclipse.wb.swt.SWTResourceManager;

import eu.DECIDEh2020.architect.appManager.models.AppDescriptor;
import eu.DECIDEh2020.architect.plugin.layoutComponents.MicroserviceListComposite;
import eu.DECIDEh2020.architect.plugin.layoutComponents.NfrListViewer;
import eu.DECIDEh2020.architect.plugin.layoutComponents.PageCompleteListener;

import java.util.ArrayList;
import java.util.HashSet;

import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;

public class ConfigPage extends WizardPage {
	
	private Composite container;
	private AppDescriptor project;
	private MicroserviceListComposite microserviceList;
	private Text txtProjectdescription;
	
	
	


	/**
	 * @wbp.parser.constructor
	 */
	public ConfigPage( AppDescriptor project) {
		super("Project Configuration");
		// TODO Auto-generated constructor stub
		this.project=project;
	}



	@Override
	public void createControl(Composite parent) {
		container = new Composite(parent, SWT.NULL);
		setControl(container);
		container.setLayout(new GridLayout(1, false));

		 Composite compositeNFR = new Composite(container, SWT.NONE);
	        compositeNFR.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
	        GridLayout gl_compositeNFR = new GridLayout(2, false);
	        gl_compositeNFR.horizontalSpacing = 15;
	        gl_compositeNFR.verticalSpacing = 15;
	        compositeNFR.setLayout(gl_compositeNFR);
	        
	        Label lblNFRs = new Label(compositeNFR, SWT.NONE);
	        lblNFRs.setLayoutData(new GridData(SWT.FILL, SWT.TOP, false, false, 1, 1));
	        lblNFRs.setText("Project NFRs:     ");
	        
	        NfrListViewer nfrListViewer = new NfrListViewer(compositeNFR, SWT.BORDER | SWT.V_SCROLL| SWT.MULTI);
	        
	        GridData gd_listNfrs = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
	        gd_listNfrs.heightHint = 75;
	        nfrListViewer.setLayoutData(gd_listNfrs);
	        
	        nfrListViewer.addSelectionListener(new SelectionAdapter() {
	        	@Override
	        	public void widgetSelected(SelectionEvent e) {
	        		HashSet<Resource> pNfrs = new HashSet<Resource>();
	        		for (Resource r: nfrListViewer.getSelection()) {
	        			try {
	        				pNfrs.add(r);
	        			} catch (IllegalArgumentException e1) {
	        				// TODO Auto-generated catch block
	        				e1.printStackTrace();
	        			}
	        		}
	        		project.setProjectNfrs(pNfrs);
	        	}
	        });
	        if(project.getProjectNfrs().size() != 0) {
				nfrListViewer.addSelection(new ArrayList<Resource>(project.getProjectNfrs()));
			}
	        addMicroserviceControls();
		
		setPageComplete(false);
	}
	
	private void addMicroserviceControls(){
		
		Composite compositeProjectDescription = new Composite(container, SWT.NONE);
		compositeProjectDescription.setLayout(new GridLayout(2, false));
		compositeProjectDescription.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Label lblAppDescription = new Label(compositeProjectDescription, SWT.NONE);
		lblAppDescription.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, false, 1, 1));
		lblAppDescription.setText("App Description:");
		
		txtProjectdescription = new Text(compositeProjectDescription, SWT.BORDER | SWT.MULTI);
		txtProjectdescription.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				project.setDescription(txtProjectdescription.getText());
			}
		});
		txtProjectdescription.setText("ProjectDescription");
		GridData gd_txtProjectdescription = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_txtProjectdescription.heightHint = 52;
		txtProjectdescription.setLayoutData(gd_txtProjectdescription);
		new Label(compositeProjectDescription, SWT.NONE);
		
	/*	Button btnMonolithic = new Button(compositeProjectDescription, SWT.CHECK);
		btnMonolithic.setText("monolithic");*/
		
		Group grpMicroservices = new Group(container, SWT.NONE);
       
        grpMicroservices.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
        grpMicroservices.setText("Microservices");
        grpMicroservices.setLayout(new GridLayout(1, false));
        
		microserviceList = new MicroserviceListComposite(grpMicroservices,SWT.NONE, project, true,SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
		microserviceList.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		microserviceList.addPageCompleteListener(new PageCompleteListener() {
			@Override
			public void pageCompleted(Boolean complete) {
				setPageComplete(complete);
			}
		});
		
	}
	
	
	
	public void setVisible(boolean visible) {
		 super.setVisible(visible);

		 if (visible) microserviceList.refresh();
	 }
	
}
