/*
* Copyright (c) 2017 Fraunhofer FOKUS.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the
* Eclipse Public License version 2.0 which accompanies
* this distribution, and is available at
* https://opensource.org/licenses/EPL-2.0
*
* Contributors:
*
* Torben Jastrow (Fraunhofer FOKUS) 
* 
* **Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu */
package eu.DECIDEh2020.architect.plugin.descriptorWizard;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;
import org.eclipse.core.runtime.*;
import java.net.URI;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.ui.*;

import eu.DECIDEh2020.architect.appManager.models.AppDescriptor;
import eu.DECIDEh2020.architect.appManager.persistence.GitHandler.Repository;


/**
 * This is a sample new wizard. Its role is to create a new file resource in the
 * provided container. If the container resource (a folder or a project) is
 * selected in the workspace when the wizard is opened, it will accept it as the
 * target container. The wizard creates one file with the extension "mpe". If a
 * sample multi-page editor (also available as a template) is registered for the
 * same extension, it will be able to open it.
 */

public class NewProjectWizard extends Wizard implements INewWizard, IExecutableExtension {
	@SuppressWarnings("unused")
	private ISelection selection;
	@SuppressWarnings("unused")
	private IConfigurationElement _configurationElement;
	private MicroservicePage microservicePage;
	private NewProjectPage projectPage;
	private ConfigPage configPage;
	private MonolithPage monolithPage;
	private AppDescriptor project;

	/**
	 * Constructor for SampleNewWizard.
	 */
	public NewProjectWizard() {
		super();
		setNeedsProgressMonitor(true);
		setWindowTitle("DECIDE project wizard");
		project = new AppDescriptor();
	}

	public AppDescriptor getProject() {
		return project;
	}

	/**
	 * Adding the page to the wizard.
	 */

	public void addPages() {
		super.addPages();

		projectPage = new NewProjectPage("From Scratch Project Wizard", project);
		projectPage.setTitle("New DECIDE Project");
		projectPage.setDescription("Create a new DECIDE project.");
		addPage(projectPage);

		configPage = new ConfigPage(project);
		addPage(configPage);

		microservicePage = new MicroservicePage(project);
		addPage(microservicePage);
		monolithPage = new MonolithPage();
		addPage(monolithPage);

	}

	/**
	 * This method is called when 'Finish' button is pressed in the wizard. We will
	 * create an operation and run it using wizard as execution context.
	 */
	public boolean performFinish() {

		final String name = projectPage.getProjectName();
		URI loc = null;
		if (!projectPage.useDefaults()) {
			loc = projectPage.getLocationURI();
			System.err.println("location: " + loc.toString()); //$NON-NLS-1$
		} // else location == null
		final URI location = loc;
		final Repository repo = projectPage.getRepository();
		
		final NewProjectWizard wizard = this;
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					NewProjectSupport.createProject(name, location, monitor, repo, wizard);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			StringWriter sw = new StringWriter();
			realException.printStackTrace(new PrintWriter(sw));
			String exceptionAsString = sw.toString();
			MessageDialog.openError(getShell(), "Error", exceptionAsString);
			return false;
		}

		
		

		BasicNewProjectResourceWizard.updatePerspective(_configurationElement);

		return true && project.getMicroservices().size() > 0;
	}

	
	
	@Override
	public IWizardPage getNextPage(IWizardPage currentPage) {

		
		if (currentPage == configPage) {
			
			microservicePage.setDescription("set Microservices: ");
			microservicePage.setMicroservices();
			return microservicePage;
		} else if (currentPage == projectPage) {
			Repository repo = projectPage.getRepository();
			if (!repo.isNewRepo()) {
				configPage.setPageComplete(true);
				if(!canFinish()) {
				getContainer().updateButtons();}
				//System.out.println("Config Page Complete: "+configPage.isPageComplete());
				
				return null;
			}else {
				configPage.setPageComplete(project!=null && !project.getMicroservices().isEmpty());
				
				//System.out.println("Config Page Complete: "+configPage.isPageComplete());
				return configPage;
			}
			
		}
		return null;
	}
	

	/**
	 * We will initialize file contents with a sample text.
	 */

	/**
	 * We will accept the selection in the workbench to see if we can initialize
	 * from it.
	 * 
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
		IPerspectiveRegistry ipr = workbench.getPerspectiveRegistry();
		for (IPerspectiveDescriptor ipd : ipr.getPerspectives()) {
			//System.out.println("Label: " + ipd.getLabel() + " - ID: " + ipd.getId() + " - Description: " + ipd.getDescription());
		}
		
	}

	@Override
	public void setInitializationData(IConfigurationElement config, String propertyName, Object data)
			throws CoreException {

		_configurationElement = config;

	}
}